﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using System.Collections.ObjectModel;
using Telerik.Windows.DragDrop;
using System.Windows.Media.Imaging;

namespace TestingSL
{
	public partial class MainPage : UserControl
	{
		private SolidColorBrush brush;
		private RadTabItem draggingItem;
		private List<RadTabItem> animationItems;

		public MainPage()
		{
			InitializeComponent();

			this.animationItems = new List<RadTabItem>();

			DragDropManager.AddDragInitializeHandler(this.myTab, OnDragInit, true);
			DragDropManager.AddDragOverHandler(this.myTab, OnDragOver, true);
			DragDropManager.AddDragDropCompletedHandler(this.myTab, OnDragCompleted, true);

			this.myTab.ItemContainerGenerator.StatusChanged += this.ItemContainerGenerator_StatusChanged;



			this.brush = new SolidColorBrush(Color.FromArgb(
															System.Convert.ToByte("ff", 16),
															System.Convert.ToByte("ff", 16),
															System.Convert.ToByte("c9", 16),
															System.Convert.ToByte("2b", 16)
														  ));
		}

		private void OnDragCompleted(object sender, DragDropCompletedEventArgs e)
		{
			this.draggingItem.Opacity = 1;
		}

		private void OnDragOver(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
		{
			this.draggingItem.Opacity = 0;
			var currentMousePosition = e.GetPosition(this);
			this.DragItem(currentMousePosition);
			e.Handled = true;
		}

		private void DragItem(Point currentMousePosition)
		{
			var items = VisualTreeHelper.FindElementsInHostCoordinates(currentMousePosition, this);
			var itemToSwap = items.OfType<RadTabItem>().Where(i => i != null && i != this.draggingItem && !this.animationItems.Contains(i)).FirstOrDefault();

			if (itemToSwap != null)
			{
				this.SwapWith(itemToSwap);
			}
		}

		private void SwapWith(RadTabItem itemToSwap)
		{
			var firstPos = itemToSwap.TransformToVisual(this).Transform(new Point());
			var secondPos = this.draggingItem.TransformToVisual(this).Transform(new Point());

			var offsetX = firstPos.X - secondPos.X;

			var translate = new TranslateTransform();
			itemToSwap.RenderTransform = translate;
			translate.X += offsetX;

			this.ChangeIndexes(itemToSwap);

			this.AnimateItemPosition(itemToSwap);
		}

		private void ChangeIndexes(RadTabItem itemToSwap)
		{
			int draggingIndex = this.myTab.Items.IndexOf(this.draggingItem);
			int swapIndex = this.myTab.Items.IndexOf(itemToSwap);

			this.myTab.Items.Remove(itemToSwap);
			this.myTab.Items.Remove(this.draggingItem);

			if (draggingIndex < swapIndex)
			{
				this.myTab.Items.Insert(draggingIndex, itemToSwap);
				this.myTab.Items.Insert(swapIndex, this.draggingItem);
			}
			else
			{
				this.myTab.Items.Insert(swapIndex, this.draggingItem);
				this.myTab.Items.Insert(draggingIndex, itemToSwap);
			}

			this.myTab.SelectedItem = this.draggingItem;
		}

		private void AnimateItemPosition(RadTabItem itemToSwap)
		{
			Storyboard sb = new Storyboard();

			DoubleAnimation animation = new DoubleAnimation();
			animation.Duration = new Duration(TimeSpan.FromMilliseconds(500));
			animation.To = 0;

			Storyboard.SetTargetProperty(animation, new PropertyPath("(UIElement.RenderTransform).(TranslateTransform.X)"));
			Storyboard.SetTarget(animation, itemToSwap);

			sb.Children.Add(animation);
			this.animationItems.Add(itemToSwap);

			sb.Completed += (_, __) =>
			{
				this.animationItems.Remove(itemToSwap);
				sb = null;
			};

			sb.Begin();
		}

		private void OnDragInit(object sender, DragInitializeEventArgs e)
		{
			var tabItem = e.OriginalSource as RadTabItem;
			if (tabItem == null)
				return;

			var draggingImage = new System.Windows.Controls.Image
			{
				Source = new Telerik.Windows.Media.Imaging.RadBitmap(tabItem).Bitmap,
				Width = tabItem.ActualWidth,
				Height = tabItem.ActualHeight
			};

			Border border = new Border() { BorderThickness = new Thickness(0, 0, 0, 1), Padding = new Thickness(0, 0, 0, 2), CornerRadius = new CornerRadius(2), BorderBrush = this.brush, Width = tabItem.ActualWidth, Height = tabItem.ActualHeight };
			border.Child = draggingImage;

			e.DragVisual = border;
			this.draggingItem = tabItem;
			e.Handled = true;
		}

		private void ItemContainerGenerator_StatusChanged(object sender, EventArgs e)
		{
			var generator = sender as Telerik.Windows.Controls.ItemContainerGenerator;
			if (generator != null && generator.Status == Telerik.Windows.Controls.Primitives.GeneratorStatus.ContainersGenerated)
			{
				this.Dispatcher.BeginInvoke(new Action(() =>
				{
					foreach (var item in this.myTab.Items)
					{
						var container = generator.ContainerFromItem(item);
						if (container != null)
							DragDropManager.SetAllowDrag(container, true);
					}
				}));
			}
		}
	}
}
